const productModel = require('../models/productModel');
const categoryModel = require('../models/categoryModel');
const orderModel = require('../models/orderModel');
const db = require('../config/db');

// แสดงสินค้าทั้งหมด
exports.getAllProducts = async (req, res) => {
    try {
        // ดึงข้อมูล categories
        const categories = await db.query('SELECT * FROM categories');
        
        // ดึงข้อมูลสินค้าทั้งหมด
        const products = await db.query('SELECT * FROM products');

        res.render('index', {
            products,
            categories,
            searchQuery: '',
            categoryId: '',
            message: null
        });
    } catch (error) {
        console.error('Error fetching products:', error);
        res.status(500).send('Error fetching products');
    }
};

// สร้างสินค้าใหม่
exports.createProduct = async (req, res) => {
    try {
        const { name, quantity, price, categoryId } = req.body;
        
        // เพิ่มสินค้าใหม่
        await db.query(
            'INSERT INTO products (name, quantity, price, category_id) VALUES (?, ?, ?, ?)',
            [name, quantity, price, categoryId]
        );
        
        // กลับไปหน้าหลัก
        res.redirect('/products');
    } catch (error) {
        console.error('Error creating product:', error);
        res.status(500).send('Error creating product');
    }
};

// ฟังก์ชันดึงประเภทสินค้า
exports.getAddProductPage = async (req, res) => {
    try {
        // ดึงข้อมูล categories สำหรับ dropdown
        const categories = await db.query('SELECT * FROM categories');
        
        res.render('addProduct', { categories });
    } catch (error) {
        console.error('Error fetching categories:', error);
        res.status(500).send('Error fetching categories');
    }
};

// ลบสินค้า
exports.deleteProduct = async (req, res) => {
    try {
        const productId = req.params.id;
        
        // ลบสินค้า
        await db.query('DELETE FROM products WHERE id = ?', [productId]);
        
        // กลับไปหน้าหลัก
        res.redirect('/products');
    } catch (error) {
        console.error('Error deleting product:', error);
        res.status(500).send('Error deleting product');
    }
};

// ค้นหาสินค้า
exports.searchProduct = async (req, res) => {
    try {
        const searchQuery = req.query.q || '';
        const categoryId = req.query.categoryId || '';

        // ดึงข้อมูล categories
        const categories = await db.query('SELECT * FROM categories');

        // สร้าง query สำหรับค้นหา
        let query = 'SELECT * FROM products WHERE 1=1';
        const params = [];

        if (searchQuery) {
            query += ' AND (name LIKE ? OR id LIKE ?)';
            params.push(`%${searchQuery}%`, `%${searchQuery}%`);
        }

        if (categoryId) {
            query += ' AND category_id = ?';
            params.push(categoryId);
        }

        // ดึงข้อมูลสินค้าตามเงื่อนไข
        const products = await db.query(query, params);

        res.render('index', {
            products,
            categories,
            searchQuery,
            categoryId,
            message: products.length === 0 ? 'ไม่พบสินค้าที่ค้นหา' : null
        });
    } catch (error) {
        console.error('Error searching products:', error);
        res.status(500).send('Error searching products');
    }
};

// สั่งสินค้า
exports.placeOrder = async (req, res) => {
    try {
        const productId = req.params.id;
        const quantity = parseInt(req.body.quantity);

        // ตรวจสอบข้อมูล
        if (!productId || !quantity || quantity <= 0) {
            return res.status(400).send('Invalid order data');
        }

        // สร้าง order
        await orderModel.createOrder(productId, quantity);

        // ไปยังหน้า success
        res.redirect('/orders/success');
    } catch (error) {
        console.error('Error placing order:', error);
        if (error.message === 'Insufficient stock') {
            res.status(400).send('Not enough stock available');
        } else if (error.message === 'Product not found') {
            res.status(404).send('Product not found');
        } else {
            res.status(500).send('Error placing order');
        }
    }
};

// หน้าแก้ไขสินค้า
exports.getEditProductPage = async (req, res) => {
    try {
        const productId = req.params.id;
        
        // ดึงข้อมูลสินค้าที่ต้องการแก้ไข
        const products = await db.query('SELECT * FROM products WHERE id = ?', [productId]);
        
        if (products.length === 0) {
            return res.status(404).send('Product not found');
        }

        const product = products[0];

        // ดึงข้อมูล categories สำหรับ dropdown
        const categories = await db.query('SELECT * FROM categories');

        res.render('editProduct', {
            product,
            categories
        });
    } catch (error) {
        console.error('Error fetching product:', error);
        res.status(500).send('Error fetching product');
    }
};

// อัพเดทข้อมูลสินค้า
exports.updateProduct = async (req, res) => {
    try {
        const productId = req.params.id;
        const { name, quantity, price, categoryId } = req.body;

        await db.query(
            'UPDATE products SET name = ?, quantity = ?, price = ?, category_id = ? WHERE id = ?',
            [name, quantity, price, categoryId, productId]
        );

        res.redirect('/products');
    } catch (error) {
        console.error('Error updating product:', error);
        res.status(500).send('Error updating product');
    }
}; 