const db = require('../config/db');

// ดึงข้อมูลสินค้าทั้งหมด
const getAllCategories = (callback) => {
  const query = 'SELECT * FROM categories';
  db.query(query, (err, result) => {
    if (err) {
      console.error(err);
      return callback(err, null);
    }
    callback(null, result);  // ส่งผลลัพธ์กลับไป
  });
};

// ค้นหาสินค้า
const searchProduct = (searchQuery, categoryId, callback) => {
  // ถ้า searchQuery ไม่ได้พิมพ์ จะค้นหาทั้งหมด
  searchQuery = String(searchQuery || '');

  let query = 'SELECT * FROM products WHERE name LIKE ?';
  let params = [`%${searchQuery}%`];

  // ถ้ามีการเลือก category ให้ค้นหาจาก category_id ด้วย
  if (categoryId) {
    query += ' AND category_id = ?';
    params.push(Number(categoryId) || null);  // ถ้า categoryId ไม่ใช่ตัวเลขให้เป็น null
  }

  // ใช้ db.query และส่ง callback ที่ถูกต้องไป
  db.query(query, params, (err, rows) => {
    if (err) {
      console.error(err);
      return callback(err, null);  // ส่ง error กลับ
    }
    callback(null, rows);  // ส่งผลลัพธ์ rows กลับไปยัง callback
  });
};

// สั่งสินค้า
const placeOrder = (product_id, quantity, callback) => {
  // ค้นหาสินค้าในคลัง
  const query = 'SELECT * FROM products WHERE id = ?';
  db.query(query, [product_id], (err, result) => {
    if (err) return callback(err);
    if (result.length === 0) return callback(new Error('Product not found'));
    const product = result[0];
    if (product.quantity < quantity) {
      return callback(new Error('Not enough stock available'));
    }
    // ลดจำนวนสินค้าในคลัง
    const updateQuery = 'UPDATE products SET quantity = quantity - ? WHERE id = ?';
    db.query(updateQuery, [quantity, product_id], (err, updateResult) => {
      if (err) return callback(err);
      // บันทึกคำสั่งซื้อ (ต้องมีตาราง orders ในฐานข้อมูล)
      const insertQuery = 'INSERT INTO orders (product_id, quantity) VALUES (?, ?)';
      db.query(insertQuery, [product_id, quantity], callback);
    });
  });
};

// สร้างสินค้าใหม่
const createProduct = (name, quantity, price, categoryId, callback) => {
  const query = `INSERT INTO products (name, quantity, price, category_id) VALUES (?, ?, ?, ?)`;
  db.query(query, [name, quantity, price, categoryId], (err, result) => {
    if (err) {
      console.error(err);
      return callback(err);
    }
    callback(null, result);
  });
};

// ลบสินค้า
const deleteProductCascade = (id, callback) => {
  // ลบข้อมูลที่เกี่ยวข้องในตาราง orders ก่อน
  const deleteOrdersQuery = 'DELETE FROM orders WHERE product_id = ?';
  db.query(deleteOrdersQuery, [id], (err) => {
    if (err) {
      console.error('Error deleting related orders:', err);
      return callback(err);
    }
    
    // จากนั้นลบสินค้า
    const deleteProductQuery = 'DELETE FROM products WHERE id = ?';
    db.query(deleteProductQuery, [id], (err, result) => {
      if (err) {
        console.error('Error deleting product:', err);
        return callback(err);
      }
      callback(null, result);
    });
  });
};

// อัพเดทข้อมูลสินค้า
const updateProduct = (id, name, quantity, price, categoryId, callback) => {
  const query = `UPDATE products SET name = ?, quantity = ?, price = ?, category_id = ? WHERE id = ?`;
  db.query(query, [name, quantity, price, categoryId, id], (err, result) => {
    if (err) {
      console.error('Error updating product:', err);
      return callback(err);
    }
    callback(null, result);
  });
};

// ดึงข้อมูลสินค้าตาม ID
const getProductById = (id, callback) => {
  const query = 'SELECT * FROM products WHERE id = ?';
  db.query(query, [id], (err, results) => {
    if (err) {
      console.error('Error fetching product:', err);
      return callback(err);
    }
    callback(null, results[0]); // ส่งข้อมูลสินค้าตัวแรกที่พบ
  });
};

module.exports = {
  getAllCategories,
  searchProduct,
  placeOrder,
  createProduct,
  deleteProductCascade,
  updateProduct,
  getProductById
};
